@@ -110,6 +110,9 @@ class Manager(object):
         # start the process tracker
         self._tracker.start()
 
+        # manage processes
+        self.subscribe('exit', self._on_exit)
+
         # start contollers
         for ctl in self.apps:
             ctl.start(self.loop, self)
@@ -590,6 +593,7 @@ class Manager(object):
         state = self.processes[name]
         if state.stopped:
             return
+
         state.stopped = True
 
         # notify others that all processes of the templates are beeing
@@ -653,7 +657,7 @@ class Manager(object):
         pid = self.get_process_id()
 
         # start process
-        p = state.make_process(self.loop, pid, self._on_exit)
+        p = state.make_process(self.loop, pid, self._on_process_exit)
         p.spawn()
 
         # add the process to the running state
@@ -674,6 +678,9 @@ class Manager(object):
             self._spawn_process(state)
 
     def _reap_processes(self, state):
+        if state.stopped:
+            return
+
         diff = len(state.running) - state.numprocesses
         if diff > 0:
             for i in range(diff):
@@ -700,6 +707,9 @@ class Manager(object):
                     pid=p.id, os_pid=p.pid)
 
     def _manage_processes(self, state):
+        if state.stopped:
+            return
+
         if len(state.running) < state.numprocesses:
             self._spawn_processes(state)
         self._reap_processes(state)
@@ -742,7 +752,6 @@ class Manager(object):
         event.update(ev)
         self._emitter.publish(evtype, event)
 
-
     def _apply_group_func(self, groupname, func):
         self._lock.acquire()
         if groupname not in self.groups:
@@ -770,35 +779,39 @@ class Manager(object):
         elif sig == "RESTART":
             self._restart()
 
-    def _on_exit(self, process, exit_status, term_signal):
-        # notify other that the process exited
-        ev_details = dict(name=process.name, pid=process.id,
-                exit_status=exit_status, term_signal=term_signal,
-                os_pid=process.pid)
-
-        self._publish("exit", **ev_details)
-        self._publish("proc.%s.exit" % process.name, **ev_details)
 
+    def _on_exit(self, evtype, msg):
         with self._lock:
-            # maybe uncjeck this process from the tracker
-            self._tracker.uncheck(process)
-
-            state = self.get_process_state(process.name)
-
-            # unexpected exit, remove the process from the list of
-            # running processes.
-
-            if process.id in self.running:
-                self.running.pop(process.id)
-
-            # this template has been removed, return
+            state = self.get_process_state(msg['name'])
             if not state:
                 return
 
-            state.remove(process)
-
             # eventually restart the process
             if not state.stopped:
                 # manage the template, eventually restart a new one.
+
                 if self._check_flapping(state):
                     self._manage_processes(state)
+
+    def _on_process_exit(self, process, exit_status, term_signal):
+        with self._lock:
+            # maybe uncjeck this process from the tracker
+            self._tracker.uncheck(process)
+
+            # unexpected exit, remove the process from the list of
+            # running processes.
+            if process.id in self.running:
+                self.running.pop(process.id)
+
+            state = self.get_process_state(process.name)
+            if state and state is not None:
+                # remove the process from the state
+                state.remove(process)
+
+            # notify other that the process exited
+            ev_details = dict(name=process.name, pid=process.id,
+                    exit_status=exit_status, term_signal=term_signal,
+                    os_pid=process.pid)
+
+            self._publish("exit", **ev_details)
+            self._publish("proc.%s.exit" % process.name, **ev_details)
